@@ -5,6 +5,8 @@
 #include "tracing.h"
 #include "transport.h"
 
+#include <uv.h>
+
 /* Initialize the given buffer for reading, ensure it has the given size. */
 static int init_read(struct conn *c, uv_buf_t *buf, size_t size)
 {
@@ -56,7 +58,7 @@ static void gateway_handle_cb(struct handle *req,
 {
 	struct conn *c = req->data;
 	size_t n;
-	void *cursor;
+	char *cursor;
 	uv_buf_t buf;
 	int rv;
 
