@@ -10,6 +10,7 @@
 
 #include "./lib/queue.h"
 #include "db.h"
+#include "lib/threadpool.h"
 #include "raft.h"
 
 #define SQLITE_IOERR_NOT_LEADER (SQLITE_IOERR | (40 << 8))
@@ -23,22 +24,19 @@ typedef void (*exec_cb)(struct exec *req, int status);
 typedef void (*barrier_cb)(struct barrier *req, int status);
 
 /* Wrapper around raft_apply, saving context information. */
-struct apply
-{
+struct apply {
 	struct raft_apply req; /* Raft apply request */
 	int status;            /* Raft apply result */
 	struct leader *leader; /* Leader connection that triggered the hook */
 	int type;              /* Command type */
 	union {                /* Command-specific data */
-		struct
-		{
+		struct {
 			bool is_commit;
 		} frames;
 	};
 };
 
-struct leader
-{
+struct leader {
 	struct db *db;          /* Database the connection. */
 	sqlite3 *conn;          /* Underlying SQLite connection. */
 	struct raft *raft;      /* Raft instance. */
@@ -47,8 +45,7 @@ struct leader
 	struct apply *inflight; /* TODO: make leader__close async */
 };
 
-struct barrier
-{
+struct barrier {
 	void *data;
 	struct leader *leader;
 	struct raft_barrier req;
@@ -58,8 +55,7 @@ struct barrier
 /**
  * Asynchronous request to execute a statement.
  */
-struct exec
-{
+struct exec {
 	void *data;
 	struct leader *leader;
 	struct barrier barrier;
@@ -68,6 +64,7 @@ struct exec
 	int status;
 	queue queue;
 	exec_cb cb;
+	pool_work_t work;
 };
 
 /**
