@@ -1,9 +1,12 @@
 #include "server.h"
 
 #include <errno.h>
+#include <sched.h>
 #include <stdlib.h>
+#include <sys/file.h>
 #include <sys/un.h>
 #include <time.h>
+#include <uv.h>
 
 #include "../include/dqlite.h"
 #include "client/protocol.h"
@@ -13,6 +16,7 @@
 #include "lib/addr.h"
 #include "lib/assert.h"
 #include "lib/fs.h"
+#include "lib/threadpool.h"
 #include "logger.h"
 #include "protocol.h"
 #include "roles.h"
@@ -40,9 +44,9 @@ static void state_cb(struct raft *r,
 
 	if (old_state == RAFT_LEADER && new_state != RAFT_LEADER) {
 		tracef("node %llu@%s: leadership lost", r->id, r->address);
-		QUEUE__FOREACH(head, &d->conns)
+		QUEUE_FOREACH(head, &d->conns)
 		{
-			conn = QUEUE__DATA(head, struct conn, queue);
+			conn = QUEUE_DATA(head, struct conn, queue);
 			gateway__leader_close(&conn->gateway,
 					      RAFT_LEADERSHIPLOST);
 		}
@@ -60,6 +64,7 @@ int dqlite__init(struct dqlite_node *d,
 	ssize_t count;
 
 	d->initialized = false;
+	d->lock_fd = -1;
 	memset(d->errmsg, 0, sizeof d->errmsg);
 
 	rv = snprintf(db_dir_path, sizeof db_dir_path, DATABASE_DIR_FMT, dir);
@@ -69,7 +74,7 @@ int dqlite__init(struct dqlite_node *d,
 		goto err;
 	}
 
-	rv = config__init(&d->config, id, address, db_dir_path);
+	rv = config__init(&d->config, id, address, dir, db_dir_path);
 	if (rv != 0) {
 		snprintf(d->errmsg, DQLITE_ERRMSG_BUF_SIZE,
 			 "config__init(rv:%d)", rv);
@@ -81,6 +86,7 @@ int dqlite__init(struct dqlite_node *d,
 		goto err_after_config_init;
 	}
 	registry__init(&d->registry, &d->config);
+
 	rv = uv_loop_init(&d->loop);
 	if (rv != 0) {
 		snprintf(d->errmsg, DQLITE_ERRMSG_BUF_SIZE,
@@ -88,9 +94,19 @@ int dqlite__init(struct dqlite_node *d,
 		rv = DQLITE_ERROR;
 		goto err_after_vfs_init;
 	}
+#ifdef DQLITE_NEXT
+	rv = pool_init(&d->pool, &d->loop, d->config.pool_thread_count,
+		       POOL_QOS_PRIO_FAIR);
+	if (rv != 0) {
+		snprintf(d->errmsg, DQLITE_ERRMSG_BUF_SIZE, "pool_init(): %s",
+			 uv_strerror(rv));
+		rv = DQLITE_ERROR;
+		goto err_after_loop_init;
+	}
+#endif
 	rv = raftProxyInit(&d->raft_transport, &d->loop);
 	if (rv != 0) {
-		goto err_after_loop_init;
+		goto err_after_pool_init;
 	}
 	rv = raft_uv_init(&d->raft_io, &d->loop, dir, &d->raft_transport);
 	if (rv != 0) {
@@ -110,7 +126,8 @@ int dqlite__init(struct dqlite_node *d,
 	if (rv != 0) {
 		snprintf(d->errmsg, DQLITE_ERRMSG_BUF_SIZE, "raft_init(): %s",
 			 raft_errmsg(&d->raft));
-		return DQLITE_ERROR;
+		rv = DQLITE_ERROR;
+		goto err;
 	}
 	/* TODO: expose these values through some API */
 	raft_set_election_timeout(&d->raft, 3000);
@@ -143,9 +160,9 @@ int dqlite__init(struct dqlite_node *d,
 		goto err_after_stopped_init;
 	}
 
-	QUEUE__INIT(&d->queue);
-	QUEUE__INIT(&d->conns);
-	QUEUE__INIT(&d->roles_changes);
+	queue_init(&d->queue);
+	queue_init(&d->conns);
+	queue_init(&d->roles_changes);
 	d->raft_state = RAFT_UNAVAILABLE;
 	d->running = false;
 	d->listener = NULL;
@@ -172,7 +189,12 @@ err_after_raft_io_init:
 	raft_uv_close(&d->raft_io);
 err_after_raft_transport_init:
 	raftProxyClose(&d->raft_transport);
+err_after_pool_init:
+#ifdef DQLITE_NEXT
+	pool_close(&d->pool);
+	pool_fini(&d->pool);
 err_after_loop_init:
+#endif
 	uv_loop_close(&d->loop);
 err_after_vfs_init:
 	VfsClose(&d->vfs);
@@ -199,6 +221,10 @@ void dqlite__close(struct dqlite_node *d)
 	// TODO assert rv of uv_loop_close after fixing cleanup logic related to
 	// the TODO above referencing the cleanup logic without running the
 	// node. See https://github.com/canonical/dqlite/issues/504.
+
+#ifdef DQLITE_NEXT
+	pool_fini(&d->pool);
+#endif
 	uv_loop_close(&d->loop);
 	raftProxyClose(&d->raft_transport);
 	registry__close(&d->registry);
@@ -492,7 +518,7 @@ static void raftCloseCb(struct raft *raft)
 
 static void destroy_conn(struct conn *conn)
 {
-	QUEUE__REMOVE(&conn->queue);
+	queue_remove(&conn->queue);
 	sqlite3_free(conn);
 }
 
@@ -533,6 +559,9 @@ static void stopCb(uv_async_t *stop)
 		tracef("not running or already stopped");
 		return;
 	}
+#ifdef DQLITE_NEXT
+	pool_close(&d->pool);
+#endif
 	if (d->role_management) {
 		rv = uv_timer_stop(&d->timer);
 		assert(rv == 0);
@@ -540,9 +569,9 @@ static void stopCb(uv_async_t *stop)
 	}
 	d->running = false;
 
-	QUEUE__FOREACH(head, &d->conns)
+	QUEUE_FOREACH(head, &d->conns)
 	{
-		conn = QUEUE__DATA(head, struct conn, queue);
+		conn = QUEUE_DATA(head, struct conn, queue);
 		conn__stop(conn);
 	}
 	raft_close(&d->raft, raftCloseCb);
@@ -649,7 +678,7 @@ static void listenCb(uv_stream_t *listener, int status)
 		goto err_after_conn_alloc;
 	}
 
-	QUEUE__PUSH(&t->conns, &conn->queue);
+	queue_insert_tail(&t->conns, &conn->queue);
 
 	return;
 
@@ -756,6 +785,12 @@ int dqlite_node_set_auto_recovery(dqlite_node *n, bool enabled)
 	return 0;
 }
 
+int dqlite_node_set_pool_thread_count(dqlite_node *n, unsigned thread_count)
+{
+	n->config.pool_thread_count = thread_count;
+	return 0;
+}
+
 const char *dqlite_node_errmsg(dqlite_node *n)
 {
 	if (n != NULL) {
@@ -796,6 +831,32 @@ static bool taskReady(struct dqlite_node *d)
 	return d->running;
 }
 
+#define LOCK_FILENAME "dqlite-lock"
+
+static int acquire_dir(const char *dir, int *fd_out)
+{
+	char path[PATH_MAX];
+	int fd;
+	int rv;
+
+	snprintf(path, sizeof(path), "%s/%s", dir, LOCK_FILENAME);
+	fd = open(path, O_RDWR|O_CREAT|O_CLOEXEC, S_IRUSR|S_IWUSR);
+	if (fd < 0) {
+		return DQLITE_ERROR;
+	}
+	rv = flock(fd, LOCK_EX|LOCK_NB);
+	if (rv != 0) {
+		return DQLITE_ERROR;
+	}
+	*fd_out = fd;
+	return 0;
+}
+
+static void release_dir(int fd)
+{
+	close(fd);
+}
+
 static int dqliteDatabaseDirSetup(dqlite_node *t)
 {
 	int rv;
@@ -804,14 +865,14 @@ static int dqliteDatabaseDirSetup(dqlite_node *t)
 		return 0;
 	}
 
-	rv = FsEnsureDir(t->config.dir);
+	rv = FsEnsureDir(t->config.database_dir);
 	if (rv != 0) {
 		snprintf(t->errmsg, DQLITE_ERRMSG_BUF_SIZE,
 			 "Error creating database dir: %d", rv);
 		return rv;
 	}
 
-	rv = FsRemoveDirFiles(t->config.dir);
+	rv = FsRemoveDirFiles(t->config.database_dir);
 	if (rv != 0) {
 		snprintf(t->errmsg, DQLITE_ERRMSG_BUF_SIZE,
 			 "Error removing files in database dir: %d", rv);
@@ -834,27 +895,39 @@ int dqlite_node_start(dqlite_node *t)
 		goto err;
 	}
 
+	int lock_fd;
+	rv = acquire_dir(t->config.raft_dir, &lock_fd);
+	if (rv != 0) {
+		snprintf(t->errmsg, DQLITE_ERRMSG_BUF_SIZE,
+			 "couldn't lock the raft directory");
+		return rv;
+	}
+	t->lock_fd = lock_fd;
+
 	rv = maybeBootstrap(t, t->config.id, t->config.address);
 	if (rv != 0) {
 		tracef("bootstrap failed %d", rv);
-		goto err;
+		goto err_after_acquire_dir;
 	}
 
 	rv = pthread_create(&t->thread, 0, &taskStart, t);
 	if (rv != 0) {
 		tracef("pthread create failed %d", rv);
 		rv = DQLITE_ERROR;
-		goto err;
+		goto err_after_acquire_dir;
 	}
 
 	if (!taskReady(t)) {
 		tracef("!taskReady");
 		rv = DQLITE_ERROR;
-		goto err;
+		goto err_after_acquire_dir;
 	}
 
 	return 0;
 
+
+err_after_acquire_dir:
+	release_dir(t->lock_fd);
 err:
 	return rv;
 }
@@ -883,6 +956,8 @@ int dqlite_node_stop(dqlite_node *d)
 	rv = pthread_join(d->thread, &result);
 	assert(rv == 0);
 
+	release_dir(d->lock_fd);
+
 	return (int)((uintptr_t)result);
 }
 
@@ -971,12 +1046,20 @@ int dqlite_node_recover_ext(dqlite_node *n,
 		};
 	}
 
+	int lock_fd;
+	rv = acquire_dir(n->config.raft_dir, &lock_fd);
+	if (rv != 0) {
+		goto out;
+	}
+
 	rv = raft_recover(&n->raft, &configuration);
 	if (rv != 0) {
 		rv = DQLITE_ERROR;
 		goto out;
 	}
 
+	release_dir(lock_fd);
+
 out:
 	raft_configuration_close(&configuration);
 	return rv;
@@ -1317,7 +1400,7 @@ int dqlite_server_set_auto_join(dqlite_server *server,
 	 * connect to. Once we've done that, we immediately fetch a fresh list
 	 * of cluster members that includes ID and role information, and clear
 	 * away the temporary node store cache. */
-	struct client_node_info info = {0};
+	struct client_node_info info = { 0 };
 	unsigned i;
 
 	for (i = 0; i < n; i += 1) {
@@ -1585,6 +1668,11 @@ int dqlite_server_start(dqlite_server *server)
 	struct client_context context;
 	int rv;
 
+	rv = sqlite3_threadsafe();
+	if (!(rv == 1 || rv == 2)) {
+		goto err;
+	}
+
 	if (server->started) {
 		goto err;
 	}
