@@ -12,23 +12,24 @@
 
 SUITE(client);
 
-static char *bools[] = {"0", "1", NULL};
+static char *bools[] = { "0", "1", NULL };
 
 static MunitParameterEnum client_params[] = {
-    {"disk_mode", bools},
-    {NULL, NULL},
+	{ "disk_mode", bools },
+	{ NULL, NULL },
 };
 
-struct fixture
-{
+struct fixture {
 	struct test_server server;
 	struct client_proto *client;
+	struct rows rows;
 };
 
 static void *setUp(const MunitParameter params[], void *user_data)
 {
 	struct fixture *f = munit_malloc(sizeof *f);
 	(void)user_data;
+	f->rows = (struct rows){};
 	test_heap_setup(params, user_data);
 	test_sqlite_setup(params);
 	test_server_setup(&f->server, 1, params);
@@ -46,6 +47,7 @@ static void tearDown(void *data)
 	test_sqlite_tear_down();
 	test_heap_tear_down(data);
 
+	clientCloseRows(&f->rows);
 	free(f);
 }
 
@@ -67,7 +69,7 @@ TEST(client, execWithOneParam, setUp, tearDown, 0, client_params)
 	uint32_t stmt_id;
 	uint64_t last_insert_id;
 	uint64_t rows_affected;
-	struct value param = {0};
+	struct value param = { 0 };
 	int rv;
 	(void)params;
 	PREPARE("CREATE TABLE test (n INT)", &stmt_id);
@@ -99,8 +101,8 @@ TEST(client, query, setUp, tearDown, 0, client_params)
 	uint64_t last_insert_id;
 	uint64_t rows_affected;
 	unsigned i;
-	struct rows rows;
 	(void)params;
+
 	PREPARE("CREATE TABLE test (n INT)", &stmt_id);
 	EXEC(stmt_id, &last_insert_id, &rows_affected);
 
@@ -116,9 +118,7 @@ TEST(client, query, setUp, tearDown, 0, client_params)
 	EXEC(stmt_id, &last_insert_id, &rows_affected);
 
 	PREPARE("SELECT n FROM test", &stmt_id);
-	QUERY(stmt_id, &rows);
-
-	clientCloseRows(&rows);
+	QUERY_DONE(stmt_id, &f->rows, {});
 
 	return MUNIT_OK;
 }
@@ -130,7 +130,6 @@ TEST(client, querySql, setUp, tearDown, 0, client_params)
 	uint64_t last_insert_id;
 	uint64_t rows_affected;
 	unsigned i;
-	struct rows rows;
 	(void)params;
 	EXEC_SQL("CREATE TABLE test (n INT)", &last_insert_id, &rows_affected);
 
@@ -142,10 +141,7 @@ TEST(client, querySql, setUp, tearDown, 0, client_params)
 	}
 
 	EXEC_SQL("COMMIT", &last_insert_id, &rows_affected);
-
-	QUERY_SQL("SELECT n FROM test", &rows);
-
-	clientCloseRows(&rows);
+	QUERY_SQL_DONE("SELECT n FROM test", &f->rows, {});
 
 	return MUNIT_OK;
 }
